import React, {Component, Fragment} from 'react';
import {MultiGrid, ScrollSync} from 'react-virtualized';
import {Checkbox, TextField, Tooltip, Zoom, Badge, Select, MenuItem, Snackbar, SnackbarContent, IconButton} from '@material-ui/core';
import {ArrowDownward as ArrowDownwardIcon, ArrowUpward as ArrowUpwardIcon, Close as CloseIcon} from '@material-ui/icons';
import {arrayMove, SortableContainer, SortableElement} from 'react-sortable-hoc';
import NumberFormat from 'react-number-format';
import {setSize, selectableRows, setColumnsWidth} from './DatatableBodyFunctions';
import { DatePicker } from 'material-ui-pickers';
import copy from 'copy-to-clipboard';
import moment from 'moment';
import 'moment/min/locales';
import width from 'text-width';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { withStyles } from '@material-ui/core/styles';

class DatatableBody extends Component {
  constructor(props) {
    super(props);
    
    let locale = window.navigator.userLanguage || window.navigator.language;
    moment.locale(locale);
    let localeData = moment.localeData();
    let dateFormat = localeData.longDateFormat('L');
    let regMask = new RegExp('[a-zA-Z]');
    let dateMask = dateFormat.split("").map(char => {
      if(regMask.test(char)){
        return /\d/;
      }else{
        return char;
      }
    });
    
    this.state = {
      classes: props.classes,
      bodyScrollbarPresent: false,
      headerScrollbarPresent: false,
      rowsCurrentPage: [],
      initialRows: [],
      filteredRows: [],
      editedRows: [],
      searchTerm: '',
      checkedRows: [],
      sortBy: [],
      font: props.font,
      multiSelect: false,
      editable: false,
      strippedRows: false,
      isSorting: false,
      resetingColumns: false,
      snackBarCopyOpen: false,
      columnsToDisplay: props.columnsToDisplay,
      keyColumn: "",
      selection: {},
      rows: [],
      dateFormat: dateFormat, 
      dateMask: dateMask, 
      userConfiguration:{},
      dimensionsGiven: {
        datatable: {
          width: 0
        },
        header: {
          height: 0
        },
        body: {
          height: 0
        },
        row: {
          height: 0
        }
      },
      dimensionsComponent: {
        datatable: {
          width: 0
        },
        header: {
          height: 0
        },
        body: {
          height: 0
        },
        screen: {
          width: 0,
          height: 0
        },
        row: {
          height: 0,
        }
      }
    };
    this.bodyRef = React.createRef();
    this.headerRef = React.createRef();
  }
  
  static getDerivedStateFromProps(props, state) {
    let updatedState = {};
    let rowsCurrentPage = props.rowsCurrentPage;
    let dimensions = props.dimensions;
    let selection = props.selection;
    let checkedRows = props.checkedRows;
    let rows = props.rows;
    let filteredRows = props.filteredRows;
    let searchTerm = props.searchTerm;
    let sortBy = props.sortBy;
    let strippedRows = props.strippedRows;
    let keyColumn = props.keyColumn;
    let editedRows = props.editedRows;
    let editable = props.editable;
    let initialRows = props.initialRows;
    let columnsToDisplay = props.columnsToDisplay;
    let userConfiguration = props.userConfiguration;
    let dimensionsComponent = state.dimensionsComponent;
    let isSorting = state.isSorting;
    
    updatedState.rows = rows;
    updatedState.filteredRows = filteredRows;
    updatedState.editedRows = editedRows;    
    updatedState.initialRows = initialRows;
    
    if(!isSorting){
      updatedState.columnsToDisplay = setColumnsWidth(columnsToDisplay, dimensionsComponent);
    }else{
      updatedState.isSorting = false;
    }
    
    if(checkedRows.length === 0 && state.checkedRows.length > 0){
      updatedState.checkedRows = [];
    }
    
    if(selection.rowSelectable){
      updatedState.rowsCurrentPage = selectableRows(rowsCurrentPage);
    }else{
      updatedState.rowsCurrentPage = rowsCurrentPage;
    }
    
    if(dimensions !== state.dimensionsGiven){
      updatedState.dimensionsGiven = dimensions;
    }
    
    if(searchTerm !== state.searchTerm){
      updatedState.searchTerm = searchTerm;
    }
    
    if(strippedRows !== state.strippedRows){
      updatedState.strippedRows = strippedRows;
    }
    
    if(JSON.stringify(selection) !== JSON.stringify(state.selection)){
      updatedState.selection = selection;    
    }
    
    if(JSON.stringify(sortBy) !== JSON.stringify(state.sortBy)){
      updatedState.sortBy = sortBy;    
    }
    
    if(JSON.stringify(userConfiguration) !== JSON.stringify(state.userConfiguration)){
      updatedState.userConfiguration = userConfiguration;    
    }
    
    if(editable !== state.editable){
      updatedState.editable = editable;
    }
    
    if(keyColumn !== state.keyColumn){
      updatedState.keyColumn = keyColumn;
    }
    
    let rowsToUse = filteredRows.length > 0 ? filteredRows : rows;
    let checked = false;
    
    if(selection.selectPageRows){
      checked = (rowsCurrentPage.filter(row => row.o2DatatableRowSelected === true).length === rowsCurrentPage.length && rowsCurrentPage.length > 0) ? true : false;      
    }else if(selection.selectAllRows){
      checked = (checkedRows.length >= rowsToUse.length) ? true : false;
    }
    
    if(checked !== state.selectAll){
      updatedState.selectAll = checked;      
    }
    
    return updatedState;
  }
  
  
  /* Listen to keydown/keyup to know if user want to do quick select by pressing shift and selecting two rows. Listen to size of the windows to resize the datatable */
  componentDidMount = () => {
    window.addEventListener("keydown", this.keydown, false);
    window.addEventListener("keyup", this.keyup, false);
    window.addEventListener('resize', this.updateWindowDimensions);
    if(this.state.rows.length > 0){  
      this.props.setBodyRef(this.bodyRef);
    }
    
    if(this.state.columnsToDisplay.length > 0){
      this.props.setHeaderRef(this.headerRef);
    }
    
    this.updateWindowDimensions();
  }
  
  /* Remove all listeners */
  componentWillUnmount = () => {
    window.removeEventListener("keydown", this.keydown, false);
    window.removeEventListener("keyup", this.keyup, false);
    window.removeEventListener('resize', this.updateWindowDimensions);
  }
  
  resetColumnsToDisplay = (columnsToDisplay) => {
    if(this.state.rows.length > 0){  
      this.props.setBodyRef(this.bodyRef);
    }
    
    if(this.state.columnsToDisplay.length > 0){
      this.props.setHeaderRef(this.headerRef);
    }
  }
  
  /* Calculate the dimensions of the datatable */
  updateWindowDimensions = () => {
    let columnsToDisplay = this.state.columnsToDisplay;
    let screenWidth = window.innerWidth;
    let screenHeight = window.innerHeight;
    let dimensionsComponent = {
      datatable: {
        width: setSize(this.state.dimensionsGiven.datatable.width, screenWidth) - 8,
      },
      header: {
        height: setSize(this.state.dimensionsGiven.header.height, screenHeight)
      },
      body: {
        height: setSize(this.state.dimensionsGiven.body.height, screenHeight)
      },
      screen: {
        width: screenWidth,
        height: screenHeight
      },
      row: {
        height: setSize(this.state.dimensionsGiven.row.height, screenHeight),
      }
    };
    
    columnsToDisplay = setColumnsWidth(columnsToDisplay, dimensionsComponent);
    
    this.setState({
      columnsToDisplay: columnsToDisplay,
      dimensionsComponent: dimensionsComponent
    });
    
    
    if(this.state.rows.length > 0){  
      this.props.setBodyRef(this.bodyRef);
      this.bodyRef.recomputeGridSize();
    }
    
    if(this.state.columnsToDisplay.length > 0){
      this.props.setHeaderRef(this.headerRef);
      this.headerRef.recomputeGridSize();
    }
    
    this.props.setDimensionsComponent(dimensionsComponent);
  }
  
  keydown = (event) => {
    if(event.key === 'Shift'){
      this.setState({multiSelect: true});
    }
  }
  
  keyup = (event) => {
    if(this.state.multiSelect){
      this.setState({multiSelect: false});
    } 
  }
  
  /* Check row */
  checkRow = index => event => {
    let rowsCurrentPage = this.state.rowsCurrentPage;
    let rows = this.state.rows;
    let checkedRows = this.state.checkedRows; 
    let multiSelect = this.state.multiSelect;
    let lastElement = checkedRows[checkedRows.length - 1];
    let row = rows.filter(row => row.o2DatatableRowUuid === rowsCurrentPage[index].o2DatatableRowUuid)[0];
    if(multiSelect && lastElement !== undefined){
      let lastElementIndex = rows.map(row => row.o2DatatableRowUuid).indexOf(lastElement.o2DatatableRowUuid);
      let newElementIndex = rows.map(row => row.o2DatatableRowUuid).indexOf(row.o2DatatableRowUuid);
      
      let firstIndex = lastElementIndex > newElementIndex ? newElementIndex : lastElementIndex;
      let lastIndex = lastElementIndex < newElementIndex ? newElementIndex : lastElementIndex;
      
      for(let i = firstIndex; i <= lastIndex; i++){
        checkedRows = this.addToCheckedRows(rows[i], true);
      }
    }else{
      checkedRows = this.addToCheckedRows(row, false);
    }
    
    this.setState({rowsCurrentPage: rowsCurrentPage[index]});
    this.setState({checkedRows: checkedRows});
    this.props.setCheckedRows(checkedRows);
  };
  
  /* Add row that have been checked to an array */
  addToCheckedRows = (row, multiple) => {
    let checkedRows = this.state.checkedRows;
    if(!multiple){
      row.o2DatatableRowSelected = !row.o2DatatableRowSelected;      
    }else{
      row.o2DatatableRowSelected = true;
    }
    
    let found = checkedRows.some((el) => {
      return el.o2DatatableRowUuid === row.o2DatatableRowUuid;
    });
    
    if(!found) {
      checkedRows.push(row); 
    }else{
      checkedRows = checkedRows.filter(checkedRow => checkedRow.o2DatatableRowUuid !== row.o2DatatableRowUuid);
    }
    
    return checkedRows;
  }
  
  /* Select all the rows or all the rows of the page depending on option given */
  selectAll = () => {
    let rowsCurrentPage = [];
    let checkedRows = this.state.checkedRows;
    let selectAll = this.state.selectAll;
    let selectPageRows = this.state.selection.selectPageRows;
    let selectAllRows = this.state.selection.selectAllRows;
    let rows = this.state.rows;
    let filteredRows = this.state.filteredRows;
    let searchTerm = this.state.searchTerm;
    let rowsToUse = filteredRows.length > 0 || searchTerm !== '' ? filteredRows : rows;
    
    if(this.state.selection.selectPageRows){
      rowsCurrentPage = this.state.rowsCurrentPage.map((row, index) => row);
    }
    
    if(selectPageRows){          
      if(selectAll){
        rowsCurrentPage = this.state.rowsCurrentPage.map((row, index) => {
          row.o2DatatableRowSelected = false;
          return row;
        });
        
        for(let row of rowsCurrentPage){
          checkedRows = checkedRows.filter(checkedRow => checkedRow.o2DatatableRowUuid !== row.o2DatatableRowUuid);    
        }
      }else{
        for(let row of rowsCurrentPage){
          if(checkedRows.filter(checkedRow => checkedRow.o2DatatableRowUuid === row.o2DatatableRowUuid).length === 0){
            row.o2DatatableRowSelected = true;
            checkedRows = checkedRows.concat(row);
          }
        }      
      }
    }else if(selectAllRows){
      rowsCurrentPage = rowsCurrentPage.map(row => {
        row.o2DatatableRowSelected = selectAll ? false : true;
        return row;
      });
      
      rowsToUse = rowsToUse.map(row => {
        row.o2DatatableRowSelected = selectAll ? false : true;
        return row;
      });
      
      checkedRows = selectAll ? [] : rowsToUse;
    }
    
    this.state.columnsToDisplay.filter(column => column.id === 'selectable')[0].o2DatatableRowSelected = !this.state.columnsToDisplay.filter(column => column.id === 'selectable')[0].o2DatatableRowSelected;
    this.setState({rowsCurrentPage: rowsCurrentPage});
    this.setState({selectAll: !selectAll});
    this.setState({checkedRows: checkedRows});
    this.props.setCheckedRows(checkedRows); 
  }
  
  
  /* Add a sort condition */
  addSort = (id) => {
    this.setState({isSorting: true});
    this.props.setSortParameter(id);
  }
  
  headerCellRenderer = ({ style, columnIndex, rowIndex, key }) => {
    let selection = this.state.selection;
    let checkedRows = this.state.checkedRows;
    let sortBy = JSON.parse(this.state.sortBy);
    let columnsToDisplay = this.state.columnsToDisplay;

    let sortIndex = sortBy.map(sort => {
      if(sort.asc !== undefined){
        return sort.asc;
      }else{
        return sort.desc;
      }
    });
        
    let asc = sortBy.map(sort => {
      if(sort.asc !== undefined){
        return sort.asc;
      }else{
        return null;
      }
    }).filter(el => el !== null);
        
    let desc = sortBy.map(sort => {
      if(sort.desc !== undefined){
        return sort.desc;
      }else{
        return null;
      }
    }).filter(el => el !== null);
        
    let align = columnsToDisplay[columnIndex].inputType === "checkbox" ? "center" : "left";
    if(columnsToDisplay[columnIndex].id === 'selectable'){
      return (
        <div
          className="Datatable-body-header-cell"
          style={style}
          key={key}
          index={columnIndex}
        >
          {checkedRows.length > 0 &&            
            <Badge className={(selection.selectPageRows || selection.selectAllRows) ? "Datatable-badge" : "Datatable-badge-without-element"} color="secondary" badgeContent={checkedRows.length}>
              <div></div>
            </Badge>
          }
  
          {(selection.selectPageRows || selection.selectAllRows) 
            ? <Checkbox
                key={"selectAllCheckbox"}
                checked={this.state.selectAll}
                onChange={this.selectAll}
                color="primary"
              />
            : ''
          }
        </div>
      )
    }else{        
      return (
        <DragableCell
          className="Datatable-body-header-cell"
          style={style}
          key={key}
          index={columnIndex}
        >
          <div style={{textAlign: align, lineHeight:'40px'}}>
            <Tooltip TransitionComponent={Zoom} title="Sort">
              <span>          
                <button 
                  onClick={() => this.addSort(columnsToDisplay[columnIndex].id)}
                  style={{textAlign: align}}
                  className={"Datatable-body-header-button"}
                >
                  {columnsToDisplay[columnIndex].label}
                </button>
              </span>
            </Tooltip>                       
          </div>
          <div style={{textAlign: align}} className={"Datatable-body-header-sort"}>           
            {asc.includes(columnsToDisplay[columnIndex].id) ? <ArrowUpwardIcon /> : ''}
            {desc.includes(columnsToDisplay[columnIndex].id) ? <ArrowDownwardIcon /> : ''}
            {sortIndex.indexOf(columnsToDisplay[columnIndex].id) !== -1 ? sortIndex.indexOf(columnsToDisplay[columnIndex].id) + 1 : ''}
          </div>
        </DragableCell>
      );  
    }
  }
  
  cellRenderer = ({ style, columnIndex, rowIndex, key }) => {
    let rowsCurrentPage = this.state.rowsCurrentPage;
    let columnsToDisplay = this.state.columnsToDisplay;
    let dateFormat = this.state.dateFormat;
    let keyColumn = this.state.keyColumn;
    let editable = this.state.editable;
    let strippedRows = this.state.strippedRows;
    let classNameCell = "Datatable-body-body-cell "; 
    classNameCell += this.state.checkedRows.find(r => r.o2DatatableRowUuid === rowsCurrentPage[rowIndex].o2DatatableRowUuid) !== undefined ? "Datatable-body-row-checked" : ""; 
    classNameCell += " Datatable-cell-" + columnsToDisplay[columnIndex].id;
    classNameCell += Number(key.split("-")[0]) % 2 === 0 && strippedRows && this.state.checkedRows.find(r => r.o2DatatableRowUuid === rowsCurrentPage[rowIndex].o2DatatableRowUuid) === undefined ? " Datatable-body-cell-stripped" : "";
    let value = rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id];
    style.lineHeight = style.height.toString() + "px";
    key += "_" + (value || rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id]);
    key = key.replace(/ /g,'');
    
    if(rowIndex !== rowsCurrentPage.length && rowIndex <= rowsCurrentPage.length - 1){
      if(columnsToDisplay[columnIndex].id === 'selectable'){
        return(
          <div key={key} style={{...style}} className={classNameCell}>
            <Checkbox
              key={key}
              checked={rowsCurrentPage[rowIndex].o2DatatableRowSelected}
              onChange={this.checkRow(rowIndex)}
              color="primary"
            />
          </div>
        )
      }else{
        let paddingTop = (style.height - 32) / 2;
        let align = columnsToDisplay[columnIndex].inputType === "checkbox" ? "center" : "left";
        style.textAlign = align;
        
        if(editable && !columnsToDisplay[columnIndex].disabled && keyColumn && keyColumn.length > 0){
          let disabled = !columnsToDisplay[columnIndex].disabled && keyColumn && keyColumn.length > 0 ? false : true;
          switch (columnsToDisplay[columnIndex].inputType) {
            case "date": {
              style.lineHeight = 0;
              value = value !== "" ? value : null;
              return (
                <div key={key} style={{...style}} className={classNameCell}>
                  <div style={{height: 60} }>
                    <DatePicker
                      style={{paddingTop: paddingTop}}
                      showTodayButton
                      format={dateFormat}
                      value={value}
                      keyboard
                      mask={this.state.dateMask}
                      onChange={(date) => this.props.editCell(date, rowIndex, columnsToDisplay[columnIndex].id, "date", true)}
                    />
                  </div>
                </div>
              )
              break;
            }
            case "checkbox": {
              return (
                <div key={key} style={{...style}} className={classNameCell}>
                  <div>
                    <Checkbox
                      checked={value === 'Y' ? true : false || rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id] === 'Y' ? true : false}
                      onChange={() => this.props.editCell(value, rowIndex, columnsToDisplay[columnIndex].id, "checkbox", true)}
                      color="primary"
                    />
                  </div>
                </div>
              )
              break;
            }
            case "select": {
              let menuItems = [];
              columnsToDisplay[columnIndex].values.map(val => {
                menuItems.push(<MenuItem key={val} value={val}>{val}</MenuItem>)
              })
              return (
                <div key={key} style={{...style}} className={classNameCell}>
                  <div>
                    <Select
                      value={value || rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id]}
                      onChange={(e) => this.props.editCell(e.target.value, rowIndex, columnsToDisplay[columnIndex].id, "select", true)}
                      className={"Datatable-body-body-select"}
                    >
                      {menuItems}
                    </Select>
                  </div>
                </div>
              )
              break;
            }
            default: {
              return (
                <div key={key} style={{...style}} className={classNameCell}>
                  <div>
                    <TextField 
                      style={{paddingTop: paddingTop}}
                      type={columnsToDisplay[columnIndex].dataType} 
                      defaultValue={value || rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id]} 
                      onChange={(e) => this.props.editCell(e.target.value, rowIndex, columnsToDisplay[columnIndex].id, "", false)}
                      onBlur={(e) => this.props.editCell(e.target.value, rowIndex, columnsToDisplay[columnIndex].id, "", true)}
                      className={"Datatable-body-body-input"}
                    />
                  </div>
                </div>
              )
            }
          }
        }else{
          switch (columnsToDisplay[columnIndex].inputType) {
            case "number": {
              return (
                <div key={key} style={{...style}} className={classNameCell}>
                  <div>
                    <NumberFormat
                      value={value}
                      displayType='text'
                      thousandSeparator
                      decimalScale={columnsToDisplay[columnIndex].decimalScale}
                      fixedDecimalScale
                    />
                  </div>
                </div>
              )
              break;
            }
            case "checkbox": {
              return (
                <div key={key} style={{...style}} className={classNameCell}>
                  <div>
                    <Checkbox
                      checked={value === 'Y' ? true : false || rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id] === 'Y' ? true : false}
                      onChange={() => this.props.editCell(value, rowIndex, columnsToDisplay[columnIndex].id, "checkbox", true)}
                      color="primary"
                      disabled
                    />
                  </div>
                </div>
              )
              break;
            }
            case "date": {
              let w = width(moment(value).format(dateFormat), {
                family: this.state.font,
                size: "0.8125rem"
              });
              let b = w > (style.width - 20) ? true : false;
              return (
                <div key={key} style={{...style}} className={classNameCell} onClick={() => this.handleCopy(moment(value).format(dateFormat))}>
                    <Tooltip title={b ? value : ""} interactive TransitionComponent={Zoom} className={"Datatable-body-body-cell-text"} classes={{ tooltip: this.state.classes.tooltip }}>
                      <div style={{maxWidth: style.width}}>
                        {moment(value).format(dateFormat) || moment(rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id]).format(dateFormat) }
                      </div>
                    </Tooltip>
                </div>
              )
              break;
            }
            default: {
              let w = width(value, {
                family: this.state.font,
                size: "0.8125rem"
              });
              let b = w > (style.width - 20) ? true : false;
              
              return (
                <div key={key} style={{...style}} className={classNameCell} onClick={() => this.handleCopy(value)}>
                    <Tooltip title={b ? value : ""} interactive TransitionComponent={Zoom} className={"Datatable-body-body-cell-text"} classes={{ tooltip: this.state.classes.tooltip }}>
                      <div style={{maxWidth: style.width}}>
                        {value || rowsCurrentPage[rowIndex][columnsToDisplay[columnIndex].id]}
                      </div>
                    </Tooltip>
                </div>
              )
            }
          }
        }
      }  
    }
  };
  
  handleCopy = (e) => {
    let copyToClipboard = this.state.userConfiguration.copyToClipboard;
    if(copyToClipboard){
      copy(e);
      this.setState({snackBarCopyOpen: true});
    }
  }
  
  snackBarCopyClose = () => {
    this.setState({snackBarCopyOpen: false});
  }
  
  onSortEnd = ({ oldIndex, newIndex }) => {
    let columnsToDisplay = this.state.columnsToDisplay;
    let columns = arrayMove(columnsToDisplay, oldIndex, newIndex);
    this.props.setColumnsToDisplay(columns)
    this.setState({
      isSorting: true,
      columnsToDisplay: columns
    });
    this.bodyRef.recomputeGridSize();
    this.headerRef.recomputeGridSize();
  };  
  
  render() {
    const {rows, columnsToDisplay, snackBarCopyOpen, dimensionsComponent, rowsCurrentPage, dateFormat, selectedDate, classes} = this.state;
    return (
      <Fragment>
        {(columnsToDisplay.length === 0) &&
          <div className={"Datatable-body-no-data"} style={{height: dimensionsComponent.body.height}}>
            There is no data yet, try to refresh <span>.</span><span>.</span><span>.</span>
          </div>
        }
        
        {(rows.length === 0 && columnsToDisplay.length > 0) &&
          <ScrollSync>
            {({onScroll, scrollLeft, scrollTop}) => (
              <Fragment>
                <SortableList
                  useWindowAsScrollContainer={true}
                  axis="x"
                  lockAxis="x"
                  onSortEnd={this.onSortEnd}
                > 
                  <MultiGrid
                    ref={ref => (this.headerRef = ref)}
                    cellRenderer={({...props}) => this.headerCellRenderer({...props})}
                    columnWidth={({ index }) => columnsToDisplay[index].width}
                    columnCount={columnsToDisplay.length}
                    fixedRowCount={1}
                    rowCount={0}
                    scrollLeft={scrollLeft}
                    onScroll={onScroll}
                    rowHeight={dimensionsComponent.header.height}
                    height={dimensionsComponent.header.height}
                    width={dimensionsComponent.datatable.width}
                  />
                    
                  {rowsCurrentPage.length === 0 &&
                    <div className={"Datatable-body-no-data"} style={{height: dimensionsComponent.body.height - 1}}>
                      There is no data yet, try to refresh<span>.</span><span>.</span><span>.</span>
                    </div>                      
                  }
                </SortableList>
              </Fragment>
            )}
          </ScrollSync>
        }
        
        
        {rows.length > 0 && columnsToDisplay.length > 0 &&          
          <ScrollSync>
            {({onScroll, scrollLeft, scrollTop}) => (
              <Fragment>
                <SortableList
                  useWindowAsScrollContainer={true}
                  axis="x"
                  lockAxis="x"
                  onSortEnd={this.onSortEnd}
                > 
                  <MultiGrid
                    ref={ref => (this.headerRef = ref)}
                    cellRenderer={({...props}) => this.headerCellRenderer({...props})}
                    columnWidth={({ index }) => columnsToDisplay[index].width}
                    columnCount={columnsToDisplay.length}
                    fixedRowCount={1}
                    rowCount={0}
                    scrollLeft={scrollLeft}
                    onScroll={onScroll}
                    rowHeight={dimensionsComponent.header.height}
                    height={dimensionsComponent.header.height}
                    width={dimensionsComponent.datatable.width}
                  />
                    
                  {rowsCurrentPage.length === 0 &&
                    <div className={"Datatable-body-no-data"} style={{height: dimensionsComponent.body.height - 1}}>
                      There is no data for your research, try something else.
                    </div>                      
                  }
                          
                  <MultiGrid
                    ref={ref => (this.bodyRef = ref)}
                    cellRenderer={({...props}) => this.cellRenderer({...props})}
                    columnWidth={({ index }) => columnsToDisplay[index].width}
                    columnCount={columnsToDisplay.length}
                    rowCount={rowsCurrentPage.length}
                    scrollLeft={scrollLeft}
                    onScroll={onScroll}
                    rowHeight={dimensionsComponent.row.height}
                    height={rowsCurrentPage.length === 0 ? 1 : dimensionsComponent.body.height}
                    width={dimensionsComponent.datatable.width + 8}
                  />
                </SortableList>
              </Fragment>
            )}
          </ScrollSync>
        }
        
        <Snackbar
          anchorOrigin={{ vertical: "bottom", horizontal: "left" }}
          open={snackBarCopyOpen}
          autoHideDuration={2500}
          onClose={this.snackBarCopyClose}
          
        >
          <SnackbarContent 
            className={classes.success}
            message={<span>Cell's content has been copied to clipboard.</span>}
            action={<IconButton><CloseIcon className={classes.icon} onClick={this.snackBarCopyClose}/></IconButton>}
          />
        </Snackbar>
      </Fragment>
    );
  }
}

const styles = theme => ({
  tooltip: {
    fontSize: "0.8rem",
    marginTop: "-10px",
    textAlign: "justify",
    background: "#1f1f1f"
  },
  success: {
    backgroundColor: "#43a047",
  },
  icon: {
    color: "white",
  }
});

const DragableCell = SortableElement(({ children, ...props }) => (
  <div {...props}>{children}</div>
));

const SortableList = SortableContainer(({ children }) => {
  return <div>{children}</div>;
});


DatatableBody.propTypes = {
  classes: PropTypes.object.isRequired,
  className: PropTypes.string,
};

export default withStyles(styles)(DatatableBody);